#!/usr/bin/env python3
"""Serveur WebSocket autonome pour le monitoring système.

Ce module fournit un serveur WebSocket indépendant qui collecte et diffuse
les données de monitoring système en temps réel à tous les clients connectés.

Classes:
    StandaloneWebSocketServer: Serveur WebSocket autonome pour le monitoring

Utilisation:
    python websocket_standalone.py [--host HOST] [--port PORT]
"""
import asyncio
import json
import logging
import re
from datetime import datetime
import websockets
import psutil
import platform
import subprocess
import socket
from utils.cpu_freq import get_cpu_max_frequency as _get_cpu_max_frequency
from utils.cpu_freq import get_cpu_current_frequency as _get_cpu_current_frequency
from typing import Dict, Any, Set

# Définir des fonctions de fréquence CPU directement ici
def get_cpu_max_frequency():
    """Version simplifiée pour obtenir la fréquence max sur Windows."""
    if platform.system() != "Windows":
        return 0
        
    # Essayer d'extraire du nom du processeur
    try:
        cpu_name = platform.processor()
        
        # Chercher pattern comme "3.70GHz"
        match = re.search(r'(\d+\.?\d*)\s*GHz', cpu_name, re.IGNORECASE)
        if match:
            ghz = float(match.group(1))
            freq_mhz = ghz * 1000
            return freq_mhz  # Convertir en MHz
    except (AttributeError, ValueError):
        pass
        
    # Méthode 1: Registre Windows directement
    try:
        # Commande simple sur une ligne
        ps_cmd = "try { $mhz = (Get-ItemProperty -Path 'HKLM:\\HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0' -Name '~MHz').'~MHz'; Write-Output $mhz } catch { Write-Output '0' }"
        
        result = subprocess.run(
            ["powershell", "-NoProfile", "-Command", ps_cmd],
            capture_output=True,
            text=True
        )
        if result.returncode == 0 and result.stdout.strip():
            freq = float(result.stdout.strip())
            if freq > 0 and freq != 2500:
                return freq
    except (subprocess.SubprocessError, OSError, ValueError):
        pass
    
    # Méthode 2: ProcessorNameString depuis le registre
    try:
        ps_cmd = "(Get-ItemProperty -Path 'HKLM:\\HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0' -Name 'ProcessorNameString').ProcessorNameString"
        
        result = subprocess.run(
            ["powershell", "-NoProfile", "-Command", ps_cmd],
            capture_output=True,
            text=True
        )
        if result.returncode == 0 and result.stdout.strip():
            proc_name = result.stdout.strip()
            
            # Chercher la fréquence dans ce nom
            match = re.search(r'(\d+\.?\d*)\s*GHz', proc_name, re.IGNORECASE)
            if match:
                ghz = float(match.group(1))
                freq = ghz * 1000
                return freq
    except (subprocess.SubprocessError, OSError, ValueError):
        pass
        
    return 0

def get_cpu_current_frequency():
    """Version simplifiée pour obtenir la fréquence actuelle sur Windows."""
    if platform.system() != "Windows":
        return 0
    
    # Méthode 1: CurrentClockSpeed simple
    try:
        ps_cmd = "(Get-CimInstance Win32_Processor).CurrentClockSpeed"
        result = subprocess.run(
            ["powershell", "-NoProfile", "-Command", ps_cmd],
            capture_output=True,
            text=True
        )
        if result.returncode == 0 and result.stdout.strip():
            freq = float(result.stdout.strip())
            # Pour AMD, même 2500 peut être valide car c'est proche de 2495
            if freq > 0:
                # Si on a 2500 et que la vraie max est 2495, ajuster proportionnellement
                if freq == 2500 and get_cpu_max_frequency() == 2495:
                    freq = 2495  # Ajuster à la vraie valeur
                return freq
    except (subprocess.SubprocessError, OSError, ValueError):
        pass
    
    # Méthode 2: Performance counter avec calcul
    # Essayer différents formats de performance counter
    counter_paths = [
        "\\Processor Information(_Total)\\% Processor Performance",
        "\\Processor(_Total)\\% Processor Time",
        "\\Informations du processeur(_Total)\\% de performance du processeur",  # Version française
        "\\Processeur(_Total)\\% temps processeur"  # Version française alternative
    ]
    
    for counter_path in counter_paths:
        try:
            ps_cmd = f"(Get-Counter '{counter_path}' -ErrorAction SilentlyContinue).CounterSamples[0].CookedValue"
            result = subprocess.run(
                ["powershell", "-NoProfile", "-Command", ps_cmd],
                capture_output=True,
                text=True
            )
            
            if result.returncode == 0 and result.stdout.strip():
                try:
                    value = float(result.stdout.strip())
                    
                    # Si c'est % Processor Time, c'est l'utilisation CPU, pas la performance
                    if "Processor Time" in counter_path or "temps processeur" in counter_path:
                        # Approximation : utilisation élevée = fréquence élevée
                        if value > 50:  # CPU utilisé à plus de 50%
                            return 2495.0
                        else:
                            # Estimation basée sur l'utilisation
                            return 2495.0 * (0.5 + value / 200.0)  # Entre 50% et 100% de la fréquence max
                    else:
                        # C'est un vrai counter de performance
                        max_freq = 2495.0
                        current_freq = (value / 100.0) * max_freq
                        return current_freq
                except ValueError:
                    pass
        except (subprocess.SubprocessError, OSError, ValueError):
            pass
    
    # Si on a toujours rien, retourner au moins 2495 comme psutil retourne 2500
    return 2495.0

# Import local pour la fréquence CPU - essayer d'abord le module utils
try:
    # Remplacer nos fonctions par celles du module
    get_cpu_max_frequency = _get_cpu_max_frequency
    get_cpu_current_frequency = _get_cpu_current_frequency
except ImportError:
    # Les fonctions définies ci-dessus seront utilisées
    pass

# Pour les GPU
try:
    import GPUtil
except ImportError:
    GPUtil = None

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)


class StandaloneWebSocketServer:
    """Serveur WebSocket autonome pour diffuser les données de monitoring.
    
    Cette classe gère les connexions WebSocket, collecte les données système
    et les diffuse périodiquement à tous les clients connectés.
    
    Attributes:
        host: Adresse IP d'écoute du serveur
        port: Port d'écoute du serveur
        clients: Ensemble des clients WebSocket connectés
    """
    
    def __init__(self, host: str = "0.0.0.0", port: int = 8765, max_clients: int = 1000) -> None:
        self.host: str = host
        self.port: int = port
        self.max_clients: int = max_clients
        self.clients: Set[Any] = set()
        self._running: bool = False
        
    async def register_client(self, websocket: Any) -> None:
        """Enregistre un nouveau client."""
        # Vérifier la limite de clients
        if len(self.clients) >= self.max_clients:
            logger.warning(f"Limite de clients atteinte ({self.max_clients}). Refus de {websocket.remote_address}")
            await websocket.close(code=1008, reason="Server at capacity")
            return
            
        self.clients.add(websocket)
        logger.info(f"Nouveau client connecté: {websocket.remote_address}. Total: {len(self.clients)}")
        
        # Message de bienvenue
        await websocket.send(json.dumps({
            "type": "connection",
            "status": "connected",
            "timestamp": datetime.now().isoformat(),
            "message": "Connecté au serveur de monitoring"
        }))
        
    async def unregister_client(self, websocket: Any) -> None:
        """Désenregistre un client."""
        if websocket in self.clients:
            self.clients.remove(websocket)
            logger.info(f"Client déconnecté: {websocket.remote_address}")
            
    async def handle_client(self, websocket: Any) -> None:
        """Gère un client WebSocket."""
        await self.register_client(websocket)
        
        try:
            async for message in websocket:
                try:
                    data = json.loads(message)
                    
                    if data.get("type") == "ping":
                        await websocket.send(json.dumps({
                            "type": "pong",
                            "timestamp": datetime.now().isoformat()
                        }))
                    elif data.get("type") == "get_status":
                        await websocket.send(json.dumps({
                            "type": "status",
                            "connected_clients": len(self.clients),
                            "monitoring_active": self._running,
                            "timestamp": datetime.now().isoformat()
                        }))
                except json.JSONDecodeError:
                    await websocket.send(json.dumps({
                        "type": "error",
                        "message": "Message invalide"
                    }))
        except websockets.exceptions.ConnectionClosed:
            pass
        finally:
            await self.unregister_client(websocket)

    @staticmethod
    def get_system_data() -> Dict[str, Any]:
        """Collecte les données système directement."""
        # Mémoire
        mem = psutil.virtual_memory()
        memory_data = {
            "total": mem.total,
            "available": mem.available,
            "used": mem.used,
            "percentage": mem.percent
        }
        
        # CPU
        cpu_percent = psutil.cpu_percent(interval=0.1)
        cpu_freq = psutil.cpu_freq()
        
        # Déterminer la fréquence maximale
        freq_max = cpu_freq.max if cpu_freq else 0
        
        # Sur Windows, toujours essayer d'améliorer la détection
        if platform.system() == "Windows":
            # Méthode directe inline pour déboguer
            cpu_name = platform.processor()
            
            # Chercher la fréquence dans le nom
            match = re.search(r'(\d+\.?\d*)\s*GHz', cpu_name, re.IGNORECASE)
            if match:
                ghz = float(match.group(1))
                freq_from_name = ghz * 1000
                if freq_from_name > 0:
                    freq_max = freq_from_name
            
            # Si on a toujours 2500, essayer notre fonction
            if freq_max == 2500 and get_cpu_max_frequency:
                better_freq = get_cpu_max_frequency()
                if better_freq > 0:
                    freq_max = better_freq
        # Pour les autres OS, utiliser si la valeur semble incorrecte
        elif get_cpu_max_frequency and (freq_max == 2500 or freq_max == 0 or freq_max is None):
            better_freq = get_cpu_max_frequency()
            if better_freq > 0:
                freq_max = better_freq
        
        # Déterminer la fréquence actuelle
        freq_current = cpu_freq.current if cpu_freq else 0
        
        # Toujours utiliser notre fonction améliorée sur Windows pour la fréquence actuelle
        if platform.system() == "Windows" and get_cpu_current_frequency:
            better_current = get_cpu_current_frequency()
            if better_current > 0:
                freq_current = better_current
        # Pour les autres OS, utiliser si la valeur semble incorrecte
        elif get_cpu_current_frequency and (freq_current == 0 or freq_current is None or freq_current == freq_max):
            better_current = get_cpu_current_frequency()
            if better_current > 0:
                freq_current = better_current
        
        cpu_data = {
            "usage_percent": cpu_percent,
            "core_count": psutil.cpu_count(logical=False),
            "logical_count": psutil.cpu_count(logical=True),
            "frequency_current": freq_current,
            "frequency_max": freq_max,
            "per_core_usage": psutil.cpu_percent(interval=0.1, percpu=True)
        }
        
        # Disque
        disk = psutil.disk_usage('/')
        disk_data = {
            "total": disk.total,
            "used": disk.used,
            "free": disk.free,
            "percentage": disk.percent,
            "path": "/"
        }
        
        # OS
        os_data = {
            "os_name": platform.system(),
            "os_version": platform.version(),
            "os_release": platform.release(),
            "architecture": platform.machine(),
            "platform": platform.platform(),
            "machine": platform.machine(),
            "processor": platform.processor(),
            "python_version": platform.python_version(),
            "hostname": socket.gethostname()
        }
        
        # GPU (si disponible)
        gpu_data = None
        
        # Essayer d'abord GPUtil
        if GPUtil:
            try:
                gpus = GPUtil.getGPUs()
                if gpus:
                    gpu = gpus[0]
                    gpu_data = {
                        "name": gpu.name,
                        "driver_version": gpu.driver,
                        "memory_total": gpu.memoryTotal * 1024 * 1024,  # Convertir en bytes
                        "memory_used": gpu.memoryUsed * 1024 * 1024,
                        "memory_free": gpu.memoryFree * 1024 * 1024,
                        "memory_percentage": gpu.memoryUtil * 100,
                        "gpu_usage_percent": gpu.load * 100,
                        "temperature": gpu.temperature,
                        "power_draw": None,
                        "power_limit": None
                    }
            except (AttributeError, RuntimeError, ValueError):
                pass  # GPUtil non disponible ou erreur
        
        # Si GPUtil n'a pas fonctionné, essayer nvidia-smi directement
        if not gpu_data:
            try:
                # Obtenir les infos GPU via nvidia-smi
                result = subprocess.run(
                    ['nvidia-smi', '--query-gpu=name,driver_version,memory.total,memory.used,memory.free,utilization.gpu,temperature.gpu', 
                     '--format=csv,noheader,nounits'],
                    capture_output=True,
                    text=True
                )
                
                if result.returncode == 0:
                    # Parser la sortie CSV
                    values = result.stdout.strip().split(', ')
                    if len(values) >= 7:
                        gpu_data = {
                            "name": values[0],
                            "driver_version": values[1],
                            "memory_total": int(values[2]) * 1024 * 1024,  # MB vers bytes
                            "memory_used": int(values[3]) * 1024 * 1024,
                            "memory_free": int(values[4]) * 1024 * 1024,
                            "memory_percentage": (int(values[3]) / int(values[2])) * 100,
                            "gpu_usage_percent": float(values[5]),
                            "temperature": float(values[6]) if values[6] != 'N/A' else None,
                            "power_draw": None,
                            "power_limit": None
                        }
            except (subprocess.SubprocessError, ValueError, FileNotFoundError, IndexError):
                pass  # nvidia-smi non disponible ou erreur
        
        return {
            "memory": memory_data,
            "processor": cpu_data,
            "disk": disk_data,
            "os": os_data,
            "gpu": gpu_data
        }
    
    async def collect_and_broadcast(self) -> None:
        """Collecte et diffuse les données de monitoring."""
        logger.info("Démarrage de la collecte de données")
        
        while self._running:
            try:
                # Collecter les données
                data = self.get_system_data()
                
                # Créer le message
                message = {
                    "type": "monitoring_data",
                    "timestamp": datetime.now().isoformat(),
                    "data": {
                        "memory": data["memory"],
                        "processor": data["processor"],
                        "disk": data["disk"],
                        "system": data["os"]
                    }
                }
                
                # Ajouter GPU si disponible
                if data["gpu"]:
                    message["data"]["gpu"] = data["gpu"]
                
                # Envoyer à tous les clients de manière efficace
                if self.clients:
                    # Copier la liste pour éviter les modifications pendant l'itération
                    clients_copy = list(self.clients)
                    disconnected = set()
                    
                    # Sérialiser le message une seule fois
                    message_json = json.dumps(message)
                    
                    # Fonction pour envoyer à un client
                    async def send_to_client(client):
                        try:
                            await asyncio.wait_for(client.send(message_json), timeout=1.0)
                        except (websockets.exceptions.WebSocketException, ConnectionError, asyncio.TimeoutError):
                            disconnected.add(client)
                        except Exception as exc:
                            logger.error(f"Erreur envoi à {getattr(client, 'remote_address', 'unknown')}: {exc}")
                            disconnected.add(client)
                    
                    # Envoyer à tous les clients en parallèle
                    if clients_copy:
                        await asyncio.gather(
                            *[send_to_client(client) for client in clients_copy],
                            return_exceptions=True
                        )
                    
                    # Retirer les clients déconnectés
                    for client in disconnected:
                        await self.unregister_client(client)
                
            except (psutil.Error, ValueError, RuntimeError) as e:
                logger.error(f"Erreur lors de la collecte des données: {e}")
                
            # Attendre avant la prochaine collecte
            await asyncio.sleep(0.5)
            
            
    async def start(self) -> None:
        """Démarre le serveur WebSocket."""
        self._running = True
        
        # Démarrer la collecte de données
        collect_task = asyncio.create_task(self.collect_and_broadcast())
        
        # Créer une fonction wrapper pour gérer la signature
        async def handle_wrapper(*args: Any) -> None:
            # Gérer différentes signatures de websockets
            if len(args) == 2:
                websocket, _ = args  # path non utilisé
            else:
                websocket = args[0]
            await self.handle_client(websocket)
        
        # Démarrer le serveur WebSocket
        logger.info(f"Démarrage du serveur WebSocket sur {self.host}:{self.port}")
        
        try:
            async with websockets.serve(handle_wrapper, self.host, self.port):
                # Attendre indéfiniment
                await asyncio.Future()
        except (KeyboardInterrupt, asyncio.CancelledError):
            # Arrêt propre
            logger.info("Arrêt du serveur WebSocket...")
            self._running = False
            
            # Annuler la tâche de collecte
            collect_task.cancel()
            try:
                await collect_task
            except asyncio.CancelledError:
                pass
            
            # Fermer toutes les connexions
            for client in list(self.clients):
                try:
                    await client.close()
                except (websockets.exceptions.WebSocketException, ConnectionError):
                    pass


def main() -> None:
    """Point d'entrée principal."""
    logger.info("=" * 50)
    logger.info("Serveur WebSocket de monitoring autonome")
    logger.info("=" * 50)
    logger.info("Écoute sur ws://0.0.0.0:8765")

    server = StandaloneWebSocketServer()

    try:
        asyncio.run(server.start())
    except KeyboardInterrupt:
        logger.info("\n\n Arrêt du serveur WebSocket...")

if __name__ == "__main__":
    main()